// # Bundle function
// Takes a root decorated class and generates a Module from it
import { bundleStore } from './writers';
import Module from './classes/module';
import events from './events/events';
import groupModulesAndProviders from './util/group-modules-providers';
export default function bundle(moduleName, provider, otherProviders = []) {
    const getProvidersFrom = (t) => bundleStore.get('providers', t) || [];
    const getModulesFrom = (t) => bundleStore.get('modules', t) || [];
    const setHasProviderWithToken = (_set, token) => [..._set].filter(p => token && p.token === token).length > 0;
    let { modules: startingModules, providers: startingProviders } = groupModulesAndProviders([provider, ...otherProviders]);
    let providers = new Set();
    let modules = new Set(startingModules);
    function parseProvider(provider) {
        if (provider) {
            if (providers.has(provider) || setHasProviderWithToken(providers, provider.token)) {
                return;
            }
            providers.add(provider);
            let annotated = provider.useClass || provider.useFactory || provider;
            getModulesFrom(annotated).forEach((mod) => modules.add(mod));
            getProvidersFrom(annotated).forEach(parseProvider);
        }
    }
    startingProviders.forEach(parseProvider);
    return Module(moduleName, [...modules]).add(...events.resolve(), ...providers);
}
