import parseSelector from '../util/parse-selector';
import { providerStore, componentStore } from '../writers';
import { Providers } from './providers';
import Module from '../classes/module';
import directiveControllerFactory from '../util/directive-controller';
import { inputsMap } from '../properties/inputs-builder';
const TYPE = 'directive';
export const Directive = ({ selector, providers = [] }) => (t) => {
    if (!selector) {
        throw new Error('Directive selector must be provided');
    }
    let { name, type: restrict } = parseSelector(selector);
    if (restrict !== 'A') {
        throw new Error('@Directive selectors can only be attributes');
    }
    if (providers !== undefined && !Array.isArray(providers)) {
        throw new TypeError(`Directive providers must be an array`);
    }
    providerStore.set('name', name, t);
    providerStore.set('type', TYPE, t);
    Providers(...providers)(t);
    componentStore.set('restrict', restrict, t);
};
Module.addProvider(TYPE, (target, name, injects, ngModule) => {
    let ddo = {};
    componentStore.forEach((val, key) => {
        ddo[key] = val;
    }, target);
    if (ddo.controllerAs) {
        ddo.bindToController = inputsMap(ddo.inputs);
    }
    ngModule.directive(name, ['$injector', ($injector) => {
            ddo.link = function ($scope, $element, $attrs, $requires, $transclude) {
                return directiveControllerFactory(this, injects, target, ddo, $injector, {
                    $scope,
                    $element,
                    $attrs,
                    $transclude,
                    $requires
                });
            };
            return ddo;
        }]);
});
